//*************************************************************************************************
// Mike Gebhard 
// Agave Robotics
//
// The Ajax.js file contains a common AJAX implementation.   What makes this implementation 
// a little different is the added client Extensible Stylesheet Language Transformations (XSLT).  
// Since AJAX stands for Asynchronous JavaScript and XML, we generate an XML stream from a
// dynamic web page and transform the XML to the appropriate XHTML.  While XML and XSLT can 
// be rather verbose, this technology comes in handy for gathering groups of information  
// then displaying the information in a meaningful way on a web page.  For example, reading 
// an 3 axis accelerometer might look something like.
//
//<accelerometer>
//  <xaxis>4</xaxis> 
//  <yaxis>249</yaxis> 
//  <zaxis>71</zaxis> 
//</accelerometer>
//
// Rather than creating 3 variables or 3 page calls, simply add the XML as the page
// page response. Then use XSLT to transform the response to an XHTML element.
//
// NOTES:
//------------------------------------------------------------------------------------
// Replace the child element else we'll get a list of elements, one for each reading.
// elementToUpdate.replaceChild(resultDocument, elementToUpdate.firstChild);
//
// Use appendChild to append DOM objects.  This will create a list of elements.
//
// IE does not use the XSLTProcessor() it has it's own XSLT processor, 
// transformNode(xsl).  transformNode(xsl) is a method of the XML object.
//
// For some reason IE/Netburner returned a null getXmlHttpRequestObject().responseXML.  
// Firefox worked fine.  It has something to do with the response type.  I think that 
// IE7 + 8 do not support the overrideMimeType() method?  In IIS IE would work if the file 
// had an .xml extension. Since IE always returned a responseText object, I created 
// an XML DOM from the responseText string which works great.
//
// When doing a GET, make the URL unique so IE will not grab the page from cache.
// The file + "?t=" + new Date().getTime() stuff simply appends the time which
// makes the URL different for each request and gets around IE's aggressive caching.
//
// I created a global httpRequestObj that holds an XMLHttpRequest() or  
// ActiveXObject("Microsoft.XMLHTTP") object.  Only use httpRequestObj for processing
// an asynchronous page request/response.  IE's stack is rather shallow so you'll get 
// stack overflows if you try to reusing it.  For example, if you try to HTTP GET the XSLT 
// document using httpRequestObj in the HandleReadyStateChange handler. Firefox will handle
// the recursive calls but IE will thrown a stack error.  That's why there is a 
// separate synchronous getXmlFile(file) method.
//
//*************************************************************************************************

// Gets the browser specific XmlHttpRequest Object
// IE 7 started support for XMLHttpRequest() the other
// browsers followed suite.
// IE 5 and 6 use an Active X obejct Microsoft.XMLHTTP
function getXmlHttpRequestObject() {
    if (window.XMLHttpRequest) 
	{
        return new XMLHttpRequest();
    } 
	else if(window.ActiveXObject) 
	{
        return new ActiveXObject("Microsoft.XMLHTTP");
    } 
	else 
	{
        alert("Your Browser does not support AJAX!");
    }
}
 
//Holds the XMLHttpRequest() object
var httpRequestObj = getXmlHttpRequestObject();
var xsltFile;

//0	The request is not initialized
//1	The request has been set up
//2	The request has been sent
//3	The request is in process
//4	The request is complete

function getContent(service, transform)
{
	xsltFile = transform;
	
    if (httpRequestObj.readyState == 4 || httpRequestObj.readyState == 0) 
	{	
		// We have to make sure the response type is
		// text/xml else the return type is null when using responseXML
		// https://developer.mozilla.org/en/XMLHttpRequest
        if(httpRequestObj.overrideMimeType)
        { 
		    httpRequestObj.overrideMimeType('text/xml');
		}
		// Get the requested file.
		// true means an asynchronous call.
		// So we need a response handler -> HandleReadyStateChange
        httpRequestObj.open("GET", service + "?t=" + new Date().getTime(), true);
		
		// Response handler
        httpRequestObj.onreadystatechange = HandleReadyStateChange; 
		
		// Send null because we're using an HTTP GET
		// If we were doing an HTTP POST then send would 
		// contains arguments like "&value1=abc&value2=def"
        httpRequestObj.send();	
	
    }        	
}


////////////////////////////////////////////////////////////
// The getHttpFile function does a synchronous HTTP GET 
// request to grab an XSLT.  The function can be used to 
// GET any XML file
////////////////////////////////////////////////////////////
function getXmlFile(file)
{
    var xhttp;
    
	// Gets the browser specific XmlHttpRequest Object
	// IE 7 started support for XMLHttpRequest() the other
	// browsers followed suite.
	// IE 5 and 6 use an Active X obejct Microsoft.XMLHTTP
    if (window.XMLHttpRequest) 
	{
        xhttp =  new XMLHttpRequest();
    } 
	else if(window.ActiveXObject) 
	{
        xhttp = new ActiveXObject("Microsoft.XMLHTTP");
    } 
	else 
	{
        alert("Your Browser does not support AJAX!");
        return;
    }  
    
    // We have to make sure the response type is
	// text/xml else the return type is null when using responseXML
	// https://developer.mozilla.org/en/XMLHttpRequest
    if(xhttp.overrideMimeType)
    { 
	    xhttp.overrideMimeType('text/xml');
	}
	
	// Get the requested file.
	// true = asynchronous
	// So we need a response handler
    xhttp.open("GET", file + "?t=" + new Date().getTime(), false);
    xhttp.send();
	
	// Get the xml response as text and convert it to
	// an XML DOM.  I had to do this for IE/Netburner.
	if (window.ActiveXObject)
	{
		var xmlString = xhttp.responseText;
		xmlObj=new ActiveXObject("Microsoft.XMLDOM");
		xmlObj.async="false";
		xmlObj.loadXML(xmlString);
		return xmlObj;
	}
	else
	{
		// Get the xml response for Mozilla, Firefox, and Opera.
		return xhttp.responseXML;	
	}
}


////////////////////////////////////////////////////////////
// State change handler.  This fucntion is called after
// a page processing the request.
////////////////////////////////////////////////////////////
function HandleReadyStateChange()
{
    // 4 The request is complete
    // 200 Status = Ok
	if (httpRequestObj.readyState == 4 && httpRequestObj.status==200) 
	{
		// Create a reference to the div we want to update.
		var elementToUpdate = document.getElementById('accelDiv');
	
		var xml
		if (window.ActiveXObject)
		{
			// Get the xml response as text and convert it to
			// an XML DOM.  I had to do this for IE/Netburner. 
			var xmlString = httpRequestObj.responseText;
			xmlObj=new ActiveXObject("Microsoft.XMLDOM");
			xmlObj.async="false";
			xmlObj.loadXML(xmlString);
			xml = xmlObj;
		}
		else
		{
			// Get the xml response for Mozilla, Firefox, and Opera.
			xml = httpRequestObj.responseXML;
		}
		
		// Test xml.xml is only valid in IE
		//alert(xml.xml);  
		
		// Get the XSLT (Extensible Stylesheet Language Transformations) 		
		var xsl = getXmlFile(xsltFile);
		//alert(xsl.xml);
				
		// Transform for Mozilla, Firefox, and Opera.
		if (document.implementation && document.implementation.createDocument)
	  	{
	  	    // Start up the XSLT Processor
	  		xsltProcessor = new XSLTProcessor();
	  		
	  		// Add the XSL style sheet
	  		xsltProcessor.importStylesheet(xsl);
	  		
	  		// Execute the transform
	  		resultDocument = xsltProcessor.transformToFragment(xml, document);
	  		
	  		// Erase the existing element contents.
	  		elementToUpdate.innerHTML = "";
	  		
	  		// And transformed HTML to the DOM
	  	    elementToUpdate.appendChild(resultDocument);

	  	}
	  	// Transform for IE
		else if (window.ActiveXObject)
	  	{
			// In IE xml docs have a transformNode method
			// that takes an XSLT style sheet
	  	    elementToUpdate.innerHTML = xml.transformNode(xsl);
	  	}
	  	else
	  	{
	  	    alert("Your Browser does not support XSLT!");    
	  	}
    }
}



